%% Import AMO data

% https://www.esrl.noaa.gov/psd/data/correlation/amon.us.long.data

filename = 'amodatalong.txt';
delimiterIn = ' ';
headerlinesIn = 1;
amo = importdata(filename,delimiterIn,headerlinesIn);

data = amo.data;
yr = length(data);
y = 1856:1/12:(2018-1/12);

dT = reshape( transpose(data(:,2:13)),1, yr*12);
dT = dT(1:length(y));
dT(dT<-10) = NaN;

rm = movmean(dT,12);

time = length(dT);

%% Get spectrum AMO

freq = fft(dT);

P2 = abs(freq/time);
P1 = P2(1:time/2+1);
P1(2:end-1) = 2*P1(2:end-1);

f = 12*(0:(time/2))/time;

figure
plot(f,P1, 'b', 'Linewidth', 2) 

ax = gca; % current axes
ax.FontSize = 14;

xlim([-0.02 3])

xlabel('Year^{-1}','FontSize',16)
ylabel('Amplitude','FontSize',16)

grid on

% saveas(gcf, 'AMOSpectrum.png')

%% Plot AMO data

dTn = dT;   dTn(dTn<=0) = 0;
dTp = dT;   dTp(dTp>=0) = 0;

figure
plot(y,dTn,'r', y,dTp, 'b', 'Linewidth', 2)
hold on
plot(y,rm,'k', 'Linewidth', 2.5)
hold on
plot(y,zeros(1,time),'k', 'Linewidth', 2)

ax = gca; % current axes
ax.FontSize = 16;

xlim([y(1) y(end)])

xlabel('Year','FontSize',18)
ylabel('AMO index ({^\circ}C)','FontSize',18)

grid on

% saveas(gcf,'AMOIndex_New.png')
