function [ dydt ] = AMO_MoC(t,y,ylag1,ylag2,par)
% AMO model with two delays
%  2 variables - surface and subsurface temperature

d = size(y,1);
n = size(y,2);
dydt = ones(d,n);

if isa(par,'function_handle')
    par = par(t);
end

c = par(:,1);

h1 = 600;
h2 = 600;
h3 = 3300;
H = 4500;
g = 9.8;
f = 1e-4;
beta = 1.5e-11;
aT = 2e-4;
aS = 7e-4;
ubar = 1e-2;
deltaT = -20;
deltaS = -1.5;
L = 6500000;

dzT = -2*c/(h1+h2)*(deltaT-aS/aT*deltaS);
dyT = 2/L*(deltaT-aS/aT*deltaS);

timescale = 60*60*24*365;
lengthscale = 4000000;

a1 = -(ubar+aT*g/(2*H*f)*(h1*(h2+h3))*dyT...
    -aT*g*beta/(4*H*f^2)*(h1^2*(h2+h3))*dzT)*timescale/lengthscale;
b1 = -(aT*g/(2*H*f)*(h2*(h2+2*h3))*dyT...
    -aT*g*beta/(4*H*f^2)*(h1*h2*(h2+2*h3))*dzT)*timescale/lengthscale;
a2 = -(aT*g/(2*H*f)*(-h1^2)*dyT...
    -aT*g*beta/(2*H*f^2)*(h1^2*(h2/2+h3))*dzT)*timescale/lengthscale;
b2 = -(ubar+aT*g/(2*H*f)*(h2*(h3-h1))*dyT...
    -aT*g*beta/(2*H*f^2)*(2*h1*h2*h3+h2^2/2*(h1+h3))*dzT)*timescale/lengthscale;

lambda1 = 1/2*(a1+b2 + sqrt((a1+b2).^2-4*a1.*b2+4*a2.*b1));
lambda2 = 1/2*(a1+b2 - sqrt((a1+b2).^2-4*a1.*b2+4*a2.*b1));

c1 = [(lambda2-a1)./(lambda1-lambda2),...
   (lambda1-a1).*(lambda2-a1)./(a2*(lambda1-lambda2));
   -a2./(lambda1-lambda2),...
   -(lambda1-a1)./(lambda1-lambda2)];

c2 = [-(lambda1-a1)./(lambda1-lambda2),...
   -(lambda1-a1).*(lambda2-a1)./(a2*(lambda1-lambda2));
   a2./(lambda1-lambda2),...
   (lambda2-a1)./(lambda1-lambda2)];


c111 = c1(1,1); c211 = c2(1,1);
c112 = c1(1,2); c212 = c2(1,2);
c121 = c1(2,1); c221 = c2(2,1);
c122 = c1(2,2); c222 = c2(2,2);

eps = par(:,2);

dydt(1,:) = 1/eps*(-y(1,:)+c111*ylag1(1,:)+c112*ylag1(2,:)...
    +c211*ylag2(1,:)+c212*ylag2(2,:));
dydt(2,:) = 1/eps*(-y(2,:)+c121*ylag1(1,:)+c122*ylag1(2,:)...
    +c221*ylag2(1,:)+c222*ylag2(2,:));

end