function [ PSD ] = PSD_discrete(w,delta_t,T,Xn)
% Power Spectral Density (discrete data) for frequecy w
%   w - frequency (M x 1)
%   delta_t - timestep
%   T - length of timeseries (N)
%   Xn - data matrix (N x S)
%   PSD - spectral density (M X S)

S = size(Xn,2);
N = size(Xn,1);
M = size(w,1);

PSD = zeros(M,S);

B = repmat(exp(-1i.*w'.*delta_t),N,1).^repmat(linspace(1,N,N)',1,M);

for i = 1:1:S
    A = repmat(Xn(:,i),1,M);
    PSD(:,i) = sum(A.*B,1)';
end

PSD = delta_t.^2./T.*(abs(PSD).^2);

% Y = Xn(1,:).*exp(-1i.*w);
% 
% for n = 1:N
%     Yn = Xn(n,:).*exp(-1i.*w.*n);
%     Y = Y + Yn;
% end
% 
% PSD = delta_t.^2./T.*(abs(Y).^2);

end

