%% Set Parameters

h1 = 600;
h2 = 600;
H = 4500;
h3 = H - h1 - h2;
W = 4000000;
L = 6500000;
g = 9.8;
f = 1*10^-4;
b = 1.5*10^-11;
at = 2*10^-4;
as = 7*10^-4;
dT = -20;
dS = -1.5;
um = 1*10^-2;

c = 1;
dyT = 2/L*(dT - as/at*dS);
dzT = -2*c/(h1 + h2)*(dT - as/at*dS);

a1y = -(h1*(h2 + h3))/(2*H);
b1y = -(h2*(h2 + 2*h3))/(2*H);
a2y = (h1^2)/(2*H);
b2y = -(h2*(h3 - h1))/(2*H);

a1z = - b/(2*f)*h1*a1y;
b1z = - b/(2*f)*h1*b1y;
a2z = - b/(2*f)*(2*h1*a1y + h2*a2y);
b2z = - b/(2*f)*(2*h1*b1y + h2*b2y);

a1 = at*g/f * (a1y*dyT + a1z*dzT) - um;
b1 = at*g/f * (b1y*dyT + b1z*dzT);
a2 = at*g/f * (a2y*dyT + a2z*dzT);
b2 = at*g/f * (b2y*dyT + b2z*dzT) - um;

year = 60*60*24*365;

af = 0;

%% Scale Model

a1s = a1/W * year;
b1s = b1/W * year;
a2s = a2/W * year;
b2s = b2/W * year;

%% Parameters Delay Difference

lm = 1/2 * (a1s + b2s - sqrt((a1s - b2s)^2 + 4*a2s*b1s));
lp = 1/2 * (a1s + b2s + sqrt((a1s - b2s)^2 + 4*a2s*b1s));

wm = 1/(2*a2s) * (a1s - b2s - sqrt( (a1s - b2s)^2 + 4*a2s*b1s));
wp = 1/(2*a2s) * (a1s - b2s + sqrt( (a1s - b2s)^2 + 4*a2s*b1s));

facp = 1/(wp-wm) * 1/lp * exp(-af/lp);
facm = 1/(wp-wm) * 1/lm * exp(-af/lm);

A1p = (a1s*wp + b1s) * (-a1s + wm*a2s) * facp;
A1m = -(a1s*wm + b1s) * (-a1s + wp*a2s) * facm;
B1p = (a1s*wp + b1s) * (-b1s + wm*b2s) * facp;
B1m = -(a1s*wm + b1s) * (-b1s + wp*b2s) * facm;
A2p = (a2s*wp + b2s) * (-a1s + wm*a2s) * facp;
A2m = -(a2s*wm + b2s) * (-a1s + wp*a2s) * facm;
B2p = (a2s*wp + b2s) * (-b1s + wm*b2s) * facp;
B2m = -(a2s*wm + b2s) * (-b1s + wp*b2s) * facm;


%% Set Grid for PDE Solution

% N_list = [20,50,100,200,500,1000,2000,5000 ];
% % Nx = N_list(i)
% 
% dx = 1/Nx;
% xgrid = transpose(0:dx:1-dx);

dx = 0.0005;
xgrid = transpose(0:dx:1-dx);
Nx = 1/dx;

dt = 0.0005;
tmax = 100;
time = 0:dt:tmax;
Nt = int64(tmax/dt+1);

eps = dx;

% Initialise temperature vector
T1 = zeros(1,Nt);   T2 = zeros(1,Nt);
T1e = zeros(1,Nt);  T2e = zeros(1,Nt);

%% Create Initial Conditions

T1t0 = exp(-(xgrid-0.5).^2./(2*(0.15)^2));
T2t0 = zeros(Nx,1);

% Standard
T1t = T1t0;     T1tn = zeros(Nx,1);
T2t = T2t0;     T2tn = zeros(Nx,1);

%% Solve the PDE

tic
for t = 1:Nt

    % Shift the full vector
    T1xp = circshift(T1t,-1);       T1xp(Nx) = -T1xp(Nx);
    T2xp = circshift(T2t,-1);       T2xp(Nx) = -T2xp(Nx);

    % Standard
    T1tn = T1t + dt * ( a1s * (T1xp - T1t)/dx + b1s * (T2xp - T2t)/dx ...
            - af*T1t );
    T2tn = T2t + dt * ( b2s * (T2xp - T2t)/dx + a2s * (T1xp - T1t)/dx ...
            - af*T2t );

    T1(t) = T1tn(1);    T2(t) = T2tn(1);    % Boundary conditions
    T1t = T1tn;         T2t = T2tn;         % Update

end
toc

%% Define History Delay Equation (Initial Part of PDE Solution)

indm = floor(1/(lm*dt));
indp = floor(1/(lp*dt));

hist1 = T1(1:indm+2);
hist2 = T2(1:indm+2);

%% Run Delay Model given History (Exact)

T1d = zeros(1,Nt);
T2d = zeros(1,Nt);

T1d(1:indm+2) = hist1;
T2d(1:indm+2) = hist2;

tic
for t = indm+3:Nt
    T1d(t) = ((A2p*b1s - A1p*b2s) * T1d(t-1-indp) + (B2p*b1s - B1p*b2s) * T2d(t-1-indp) ...
            + (A2m*b1s - A1m*b2s) * T1d(t-1-indm) + (B2m*b1s - B1m*b2s) * T2d(t-1-indm)) /(a2s*b1s - a1s*b2s);
    T2d(t) = ((A1p*a2s - A2p*a1s) * T1d(t-1-indp) + (B1p*a2s - B2p*a1s) * T2d(t-1-indp) ...
            + (A1m*a2s - A2m*a1s) * T1d(t-1-indm) + (B1m*a2s - B2m*a1s) * T2d(t-1-indm)) /(a2s*b1s - a1s*b2s);
end
toc

%% Run Delay Model given History (Numerical)

T1dn = zeros(1,Nt);
T2dn = zeros(1,Nt);

T1dn(1:indm+2) = hist1;
T2dn(1:indm+2) = hist2;

tic
for t = indm+3:Nt
   T1dn(t) = T1dn(t-1) + dt/eps * (-a1s*T1dn(t-1) - b1s*T2dn(t-1) + A1p*T1dn(t-1-indp) + B1p*T2dn(t-1-indp) ...
             + A1m*T1dn(t-1-indm) + B1m*T2dn(t-1-indm) );
   T2dn(t) = T2dn(t-1) + dt/eps * (-a2s*T1dn(t-1) - b2s*T2dn(t-1) + A2p*T1dn(t-1-indp) + B2p*T2dn(t-1-indp) ...
             + A2m*T1dn(t-1-indm) + B2m*T2dn(t-1-indm) );
end
toc

%% Compute Correction Terms Delay Model

% Using the numerical solution
dT1dn = gradient(T1dn)/dt;	d2T1dn = gradient(dT1dn)/dt;
dT2dn = gradient(T2dn)/dt;	d2T2dn = gradient(dT2dn)/dt;

ed1p = ((lp+af)^2-7/6*lp^2)/(2*lp^2);   ed2p = 2*(lp+af)/(2*lp^2);     ed3p = 1/(2*lp^2);
ed1m = ((lm+af)^2-7/6*lm^2)/(2*lm^2);   ed2m = 2*(lm+af)/(2*lm^2);     ed3m = 1/(2*lm^2);

feps1 = -af * T1dn(indm+1:end) ...
    + A1p * (ed1p*T1dn(indm+1-indp:end-indp) + ed2p*dT1dn(indm+1-indp:end-indp) + ed3p*d2T1dn(indm+1-indp:end-indp)) ...
    + B1p * (ed1p*T2dn(indm+1-indp:end-indp) + ed2p*dT2dn(indm+1-indp:end-indp) + ed3p*d2T2dn(indm+1-indp:end-indp)) ...
    + A1m * (ed1m*T1dn(indm+1-indm:end-indm) + ed2m*dT1dn(indm+1-indm:end-indm) + ed3m*d2T1dn(indm+1-indm:end-indm)) ...
    + B1m * (ed1m*T2dn(indm+1-indm:end-indm) + ed2m*dT2dn(indm+1-indm:end-indm) + ed3m*d2T2dn(indm+1-indm:end-indm));

feps2 = -af * T2dn(indm+1:end) ...
    + A2p * (ed1p*T1dn(indm+1-indp:end-indp) + ed2p*dT1dn(indm+1-indp:end-indp) + ed3p*d2T1dn(indm+1-indp:end-indp)) ...
    + B2p * (ed1p*T2dn(indm+1-indp:end-indp) + ed2p*dT2dn(indm+1-indp:end-indp) + ed3p*d2T2dn(indm+1-indp:end-indp)) ...
    + A2m * (ed1m*T1dn(indm+1-indm:end-indm) + ed2m*dT1dn(indm+1-indm:end-indm) + ed3m*d2T1dn(indm+1-indm:end-indm)) ...
    + B2m * (ed1m*T2dn(indm+1-indm:end-indm) + ed2m*dT2dn(indm+1-indm:end-indm) + ed3m*d2T2dn(indm+1-indm:end-indm));

%% Plot the Correction Term

% Filter out high frequency oscillations
feps1_filt = filloutliers(feps1(1:200:end),'linear', 'movmedian',50);
feps2_filt = filloutliers(feps2(1:200:end),'linear', 'movmedian',50);

N_list = [50,100,200,500,1000,2000,5000 ];
error_f1eps = [1.01*10^(-2), 6.872*10^(-3), 4.913*10^(-3), 3.405*10^(-3), 2.314*10^(-3), 1.406*10^(-3), 6.423*10^(-4)];
error_f2eps = [3.628*10^(-3), 2.710*10^(-3), 2.091*10^(-3), 1.489*10^(-3), 1.048*10^(-3), 6.325*10^(-4), 2.973*10^(-4)];

ybnb = 0.003;

figure

subplot(2,1,1);
plot(time(indm+1:200:end), eps*feps1_filt, '-r', 'Linewidth', 2.5)
hold on
plot(time(indm+1:200:end), eps*feps2_filt, '-.b', 'Linewidth', 2.5)

title('N=2000');
ax = gca; % current axes
ax.FontSize = 18;
xlim([50 100])
xlabel('Time (year)','FontSize',20)
ylabel('error (^\circC)','FontSize',20)
% legend('f_{\epsilon 1}(t)', 'f_{\epsilon 2}(t)','Location','northeast', 'FontSize',20, 'NumColumns',1)
grid on

subplot(2,1,2);
% semilogx(N_list, error_f1eps,'r','Linewidth', 2.5)
loglog(N_list, error_f1eps,'r','Linewidth', 2.5)
hold on
% semilogx(N_list, error_f2eps,'-.b','Linewidth', 2.5)
loglog(N_list, error_f2eps,'-.b','Linewidth', 2.5)

xlim([50 5000])
ax = gca; % current axes
ax.FontSize = 18;
xlabel('N','FontSize',20)
% ylabel('max_{t}|...|','FontSize',20)
yticks([10^(-3), 10^(-2)])
legend('f_{\epsilon 1}(t)','f_{\epsilon 2}(t)', ...
        'Location','northeast','FontSize',20, 'NumColumns',2)
grid on

% saveas(gcf,'AMO_theoreticalerror_new.png')



%% Frequency Analysis

fy1 = fft(feps1);           fey1 = fft(T1 - T1dn);
fy2 = fft(feps2);           fey2 = fft(T2 - T2dn);

n = length(feps1);          ne = length(T1);
fs = 1/dt;
f = (0:n-1)*(fs/n);         fe = (0:ne-1)*(fs/ne);
p1 = abs(fy1).^2/n;         pe1 = abs(fey1).^2/ne;
p2 = abs(fy2).^2/n;         pe2 = abs(fey2).^2/ne;

figure

% plot(fe,pe1/max(pe1),'Linewidth', 2.5)
% hold on
plot(f,p1/max(p1),'r', 'Linewidth', 2.5)
% semilogy( f,p1/max(p1),'r', 'Linewidth', 2.5 )
hold on
plot(f,p2/max(p2),'-.b','Linewidth', 2.5)
% semilogy( f,p2/max(p2),'b', 'Linewidth', 2.5 )
xlim([0 1])

title('N=2000');
ax = gca; % current axes
ax.FontSize = 18;
xlabel('Frequency (years^{-1})','FontSize',20)
ylabel('Spectral Density','FontSize',20)
% legend('f_{\epsilon 1}', 'f_{\epsilon 2}', 'Location','northeast','FontSize',20)
legend('f_{\epsilon 1}', 'f_{\epsilon 2}', 'Location','southeast','FontSize',20)
grid on

% saveas(gcf,'AMO_theoreticalerror_freq_log.png')

